%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%    This file is to test the quality of the solutions under the fist metric  %%%
%%%  Consider five methods: IRRLyap-RTRNewton, IRRLyap-RNewton, KPIK, RKSM, and %%%
%%%  mess_lradi.                                                                %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;clc;close all;
seed = 100;
rng(seed);
[A, M, B, C, n, m] = GenerateData("rail_1357");
b = B(:, 1);
B = [];
LM = chol(M, "lower");

ExtParams.ext_Tol = 1e-16; 
ExtParams.p_Min = 5;
ExtParams.p_Inc = 1;
ExtParams.p_Max = 35;

% common parameters for different solvers    
IntParams.Min_Iteration = 2;
IntParams.Max_Iteration = 100;
IntParams.OutputGap = 1;
IntParams.Verbose = 2;
IntParams.Stop_Criterion = 2;

precond = 1; % 0, 1, 2, 3
gap = 0;     % 0 or 3; 
ParamSet = precond + gap;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             RTRNewton                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
IntParams.method = 'RTRNewton';
IntParams.initial_Delta = 100;
IntParams.maximum_Delta = 100000000;
IntParams.InitSteptype = 0;
IntParams.ParamSet = ParamSet;
IntParams.Tolerance = 1e-10;
IntParams.precond = precond;
scaling = 1;
sscaling = 1;
[FinalY_RTRNewton1, outInfo_RTRNewton1, rel_res_RTRNewton1] = ...
  RLyapSolver(A*scaling, M*scaling, LM*sscaling, B*scaling, b*scaling, ExtParams, IntParams);

figure(1)
tt = ExtParams.p_Min : ExtParams.p_Inc : ExtParams.p_Max;
grid on
semilogy(tt, rel_res_RTRNewton1,'-o', 'LineWidth', 2, 'MarkerSize',10);
leg = "RLyap-RTRNewton";
legend(leg,'FontSize', 15)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             RNewton                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
IntParams.method = 'RNewton';
IntParams.InitSteptype = 0;
IntParams.ParamSet = ParamSet;
IntParams.Tolerance = 1e-8;
IntParams.precond = precond;
IntParams.Num_pre_funs = 10;
[FinalY_RNewton1, outInfo_RNewton1, rel_res_RNewton1] = ...
  RLyapSolver(A*scaling, M*scaling, LM*sscaling, B*scaling, b*scaling, ExtParams, IntParams);

figure(1)
hold on
grid on
semilogy(tt, rel_res_RNewton1,'-d', 'LineWidth', 2, 'MarkerSize',10);
leg = [leg;"RLyap-RNewton"];
legend(leg,'FontSize', 15)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             KPIK                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
LM = chol(M,'lower');
m = 100;     % max space dimension, say sqrt(size(A))
tol = 1e-14;
tolY = eps;  
[Z_kpik, rel_res_kpik] = kpik(- A, M, LM, b, m, tol, tolY);

figure(1)
hold on
grid on
semilogy(tt, rel_res_kpik(tt),'-p', 'LineWidth', 2, 'MarkerSize',10);
leg = [leg;"K-PIK"];
legend(leg,'FontSize', 15)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             RKSM                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
m = 1000;
tol = 1e-14;
tolY = eps;
opts.tol = 1e-10;
s1=eigs(A,1,'largestreal',opts);
s2=eigs(A,1,'smallestreal',opts);
params.tol = tol;
params.tolY = tolY;
params.smax=s1;
params.smin=s2;
params.m=100;
params.ch=0;
params.period=1;
params.StagNum = 5;

[Z_rksm, rel_res_rksm]=RKSM(-A, M, LM, b, params);

figure(1)
hold on
grid on
semilogy(tt, rel_res_rksm(tt),'-s', 'LineWidth', 2, 'MarkerSize',10);
leg = [leg;"RKSM"];
legend(leg,'FontSize', 15)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             mess_lradi                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%mess_lyap(A, B, [], [], E)
eqn.A_ = -A;
eqn.B = b;
eqn.C = C;
eqn.G = b;
eqn.S = [];
eqn.E_ = M;
eqn.haveE = 1;
eqn.type = 'N';
opts.adi.res_tol = 1e-14;
opts.adi.maxiter = 100;
opts.adi.compute_sol_fac = 1;
% opts.adi.info = 1;
opts.shifts.num_desired = 6;
opts.shifts.method = 'heuristic';   % default is heuristic, worse
% opts.shifts.info = 1;
% set operation
clear oper;
oper = operatormanager('default');
[out_mess1] = mess_lradi(eqn, opts, oper);

figure(1)
hold on
grid on
semilogy(tt, out_mess1.res(tt),'-^', 'LineWidth', 2, 'MarkerSize',10);
leg = [leg;"mess\_lradi-heur"];
legend(leg,'FontSize', 15)

%%%%%
eqn.A_ = -A;
eqn.B = b;
eqn.C = C;
eqn.G = b;
eqn.S = [];
eqn.E_ = M;
eqn.haveE = 1;
eqn.type = 'N';
opts.adi.res_tol = 1e-14;
opts.adi.maxiter = 100;
opts.adi.compute_sol_fac = 1;
% opts.adi.info = 1;
opts.shifts.num_desired = 6;
opts.shifts.method = 'wachspress';   % default is heuristic, worse
clear oper;
oper = operatormanager('default');
[out_mess2] = mess_lradi(eqn, opts, oper);

figure(1)
hold on
grid on
semilogy(tt, out_mess2.res(tt),'-v', 'LineWidth', 2, 'MarkerSize',10);
leg = [leg;"mess\_lradi-wach"];
legend(leg,'FontSize', 15)

%%%%
eqn.A_ = -A;
eqn.B = b;
eqn.C = C;
eqn.G = b;
eqn.S = [];
eqn.E_ = M;
eqn.haveE = 1;
eqn.type = 'N';
opts.adi.res_tol = 1e-14;
opts.adi.maxiter = 100;
opts.adi.compute_sol_fac = 1;
% opts.adi.info = 1;
opts.shifts.num_desired = 6;
opts.shifts.method = 'projection';   % default is heuristic, worse
clear oper;
oper = operatormanager('default');
[out_mess3] = mess_lradi(eqn, opts, oper);

figure(1)
hold on
grid on
semilogy(tt, out_mess3.res(tt),'->', 'LineWidth', 2, 'MarkerSize',10);
leg = [leg;"mess\_lradi-proj"];
legend(leg,'FontSize', 15)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    best lower rank approximation                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Xstar = lyap(A, -b * b', [], M);
[U, S, V] = svd(Xstar);
rel_res_best = [];
normb = norm(b * b', 'fro');
for i = ExtParams.p_Min: ExtParams.p_Inc : ExtParams.p_Max
    XX = U(:,1:i) * S(1:i, 1:i) * V(:,1:i)';
    tmp = norm(A * XX * M + M * XX * A - b * b', 'fro') / normb ;
    rel_res_best = [rel_res_best; tmp];
end

figure(1)
hold on
grid on
semilogy(tt, rel_res_best,'-*', 'LineWidth', 2, 'MarkerSize',10);
leg = [leg;"best low rank"];
legend(leg,'FontSize', 15)
xlabel('rank','FontSize', 20);
ylabel('relative residual', 'FontSize', 20);

saveas(1, 'QualityofLowRankSol.fig')


