clear;clc;close all;


fprintf("========================================== Start! ========================================================\n")
K = 2;

methods = ['RNewton1np',   'RNewton1p',   'RNewton1old' ...
           'RTRNewton1np', 'RTRNewton1p', 'RTRNewton1old'];

% nppairs = {[4000, 3], [8000, 5], [20000, 8],[40000, 10]};

status = zeros(6, length(nppairs));

fvs      = zeros(6, K, length(nppairs)); 
gfvs     = zeros(6, K, length(nppairs)); 
gfgf0s   = zeros(6, K, length(nppairs)); 
iters    = zeros(6, K, length(nppairs)); 
nfs      = zeros(6, K, length(nppairs)); 
ngs      = zeros(6, K, length(nppairs));  
nRs      = zeros(6, K, length(nppairs));  
nVs      = zeros(6, K, length(nppairs)); 
nVps     = zeros(6, K, length(nppairs)); 
nHs      = zeros(6, K, length(nppairs));  
ComTimes = zeros(6, K, length(nppairs));
approx_ress = zeros(6, K, length(nppairs)); 

count = 1;
for np = nppairs

    for seed = 1:K
    rng(seed)
    
        n = np{1}(1);
        p = np{1}(2);
        
        fprintf('\nseed:%d, n:%d, p:%d\n', seed, n, p)
        
        h = 1 / (n + 1);
        A = 1 / h^2 * spdiags(ones(n, 1) * [-1 2 -1], [-1 0 1], n, n);    
%        M = spdiags([rand(n-1, 1); 0] + 0.1, [0], n, n);
        M = sparse(eye(n));
        LM = chol(M, 'lower');
        b = randn(n, 1);
        B = [];
        
        initialX.main = randn(n, p);    
        
        % parameters for the manifold
        ManiParams.name = 'SymFixedRankQ';
        ManiParams.n = n;
        ManiParams.p = p;
        
        ManiParams.IsCheckParams = 0;
        HasHHR = 0;
        % parameters for solvers    
        SolverParams.IsCheckParams = 0;
        SolverParams.IsCheckGradHess = 0;
        SolverParams.Max_Inner_Iter = 500;
        SolverParams.Min_Iteration = 1;
        SolverParams.Max_Iteration = 1000;
        SolverParams.OutputGap = 5;
        SolverParams.Verbose = 2;
        SolverParams.Tolerance = 1e-12;        
        SolverParams.Accuracy = 1e-12;
        SolverParams.DEBUG = 1;
        
        % ParamSet: 
        %   1: first metric on R^{n \times p}_*; vector transport by parallelization
        %   4: first metric on R^{n \times p}_*; vector transport by projection
        %   2: second metric on R^{n \times p}_*; vector transport by parallelization
        %   5: second metric on R^{n \times p}_*; vector transport by projection
        %   3: third metric on R^{n \times p}_*; vector transport by parallelization
        %   6: third metric on R^{n \times p}_*; vector transport by projection        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% solving problem
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        for mc = 1:6
            if mc <= 3  % RNewton                 
                SolverParams.method = 'RNewton';
                SolverParams.Num_pre_funs = 1;
                ManiParams.ParamSet = 1;
                if mc == 1 
                    preconditioner = 0;            
                elseif mc == 2
                    preconditioner = 1;
                else 
                    preconditioner = 4;
                end
            else  % RTRNewton                
                SolverParams.method = 'RTRNewton';
                SolverParams.initial_Delta = 1e10;    % TR-type
                SolverParams.maximum_Delta = 1e14;   % TR-type
                ManiParams.ParamSet = 1;
                if mc == 4
                    preconditioner = 0;            
                elseif mc == 5
                    preconditioner = 1;
                else
                    preconditioner = 4;
                end
            end            
            fprintf('prcond.:%d\n', preconditioner)
            [FinalX_, fv_, gfv_, gfgf0_, iter_, nf_, ng_, nR_, nV_, nVp_, nH_, ComTime_, funs_, grads_, times_, eigHess_] = ...
                LyapunovLR_(A, M, LM, B, b, n, p, ManiParams, SolverParams, HasHHR, initialX, preconditioner);
        
            if gfgf0_ < SolverParams.Tolerance
                status(mc, count) = status(mc, count) + 1;
                fvs(mc, seed, count)      = fv_; 
                gfvs(mc, seed, count)     = gfv_; 
                gfgf0s(mc, seed, count)   = gfgf0_; 
                iters(mc, seed, count)    = iter_; 
                nfs(mc, seed, count)      = nf_; 
                ngs(mc, seed, count)      = ng_;  
                nRs(mc, seed, count)      = nR_;  
                nVs(mc, seed, count)      = nV_; 
                nVps(mc, seed, count)     = nVp_; 
                nHs(mc, seed, count)      = nH_;
                ComTimes(mc, seed, count) = ComTime_;
                approx_ress(mc, seed, count) = FinalX_.approx_res; 
            end 
        
        end  %  for mc
        
        
        fprintf('##################################\nseed:%d, count: %d, num of succ: %d\n', seed, count, sum(status(:, count)));
        disp(reshape(status(:, count),1,6));
        
        
        clear ManiParams SolverParams
    
    end  % for seed 

    count = count + 1;

end  % for nppairs

save('test_table1_I', 'status', 'fvs', 'gfvs', 'gfgf0s', 'iters', 'nfs', 'ngs', 'nRs', 'nVs', 'nVps', 'nHs', 'ComTimes', 'approx_ress')


