%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     This file is to run four solvers, including K-PIK, RKSM, mess_lradi      %
%     IRRLyap-Newton, on three real data.                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;clc;


for data = ["rail_5177", "rail_20209", "rail_79841"]
% for data = ["rail_5177"]

time_kpik = 0;
time_rksm = 0;
time_mess = 0;
time_mixed = 0;
time_RNewton = 0;

timeOfSystems_kpik = 0;
timeOfSystems_rksm = 0;
timeOfSystems_mixed = 0;
timeOfSystems_RNewton = 0;



NNN = 5;
for i = 1:NNN

    seed = 100;
    rng(seed);

    %% Get data  

    [A, M, B, C, n, m] = GenerateData(data);  % 1357 5177 20209 79841           
    b = B(:,1);
    rhs = b;
    B = [];
    LM = chol(M, "lower");
    

    %% K-PIK
    fprintf('\nStart kpik...\n')
    m = 300;     % max space dimension, say sqrt(size(A))    
    tol = 1e-6;     
    tolY = eps;     
    [Z_kpik, rel_res_kpik, time_kpik_,timeOfSystems_kpik_] = kpik(- A, M, LM, rhs, m, tol, tolY);
    time_kpik = time_kpik + time_kpik_;
    timeOfSystems_kpik = timeOfSystems_kpik + timeOfSystems_kpik_;
    
    
    %% RKSM
    fprintf('\nStart rksm...\n') 
    tt = tic; 
    m = 1000;
    tol = 1e-6;
    tolY = eps;
    opts.tol = tol;
    s1=eigs(A,1,'largestreal',opts);
    s2=eigs(A,1,'smallestreal',opts);
    params.tol = tol;
    params.tolY = tolY;
    params.smax=s1;
    params.smin=s2;
    params.m=100;
    params.ch=0;
    params.period=1;
    params.StagNum = 5;
    % profile on
    [Z_rksm, rel_res_rksm,~,~,timeOfSystems_rksm_]=RKSM(-A, M, LM, b, params);
    time_rksm = time_rksm + toc(tt);
    timeOfSystems_rksm = timeOfSystems_rksm + timeOfSystems_rksm_;


    %% mess_lradi
    fprintf('\nStart mess_lradi...\n')    
    tt = tic;
    eqn.A_ = -A;
    eqn.B = b;
    eqn.C = [];
    eqn.G = b;
    eqn.S = [];
    eqn.E_ = M;
    eqn.haveE = 1;
    eqn.type = 'N';
    opts.adi.res_tol = 1e-6;
    opts.adi.maxiter = 100;
    opts.adi.compute_sol_fac = 1;
    % opts.adi.info = 1;
    opts.shifts.num_desired = 6;
    opts.shifts.method = 'projection';   % default is heuristic, worse
    % opts.shifts.info = 1;
    % set operation
    clear oper;    
    oper = operatormanager('default');    
    [out_mess, eqn, opts, oper] = mess_lradi(eqn, opts, oper);
    time_mess = time_mess + toc(tt);
    Z_mess_ = out_mess.Z;
    
    FF = full([A * Z_mess_,  M * Z_mess_, rhs]);
    [~, R] = qr(FF, 0);
    p = size(Z_mess_, 2);
    l = size(rhs, 2);
    T = [zeros(p, p), eye(p), zeros(p, l); eye(p), zeros(p, p), zeros(p, l); zeros(l, p), zeros(l, p), -eye(l)]; 
    rel_res_mess = norm(R * T * R', 'fro') / norm(rhs' * rhs, 'fro');
    fprintf('rel_res_mess:%.3e,CPUtime:%.3e\n',rel_res_mess,time_mess)
    
    clear enq opts oper


    %% mess_lradi + fixed rank Lyapunov solver
    fprintf('\nStart lradi-LyapLR...\n')     
    tt = tic;
    eqn.A_ = -A;
    eqn.B = b;
    eqn.C = [];
    eqn.G = b;
    eqn.S = [];
    eqn.E_ = M;
    eqn.haveE = 1;
    eqn.type = 'N';
    opts.adi.res_tol = 1e-3;
    opts.adi.maxiter = 100;
    opts.adi.compute_sol_fac = 1;    
    opts.shifts.num_desired = 6;
    opts.shifts.method = 'projection';   % default is heuristic, worse    
    % set operation
    clear oper;
    oper = operatormanager('default');    
    [out_mess, eqn, opts, oper] = mess_lradi(eqn, opts, oper);    
    Z_mess = out_mess.Z;
    
    global res_ 
    res_ = 1;

    p = size(Z_mess, 2);
%     p = estimateRank(A, 5e-5)
    % parameters for the manifold
    ManiParams.name = 'SymFixedRankQ';
    ManiParams.n = n;
    ManiParams.p = p;
    ManiParams.IsCheckParams = 0;
    ManiParams.ParamSet = 1;
    HasHHR = 0;    
    % parameters for solvers    
    SolverParams.method = 'RNewton';
    SolverParams.IsCheckParams = 0;
    SolverParams.IsCheckGradHess = 0;
    SolverParams.Min_Iteration = 1;
    SolverParams.Max_Iteration = 100;
    SolverParams.OutputGap = 1;
    SolverParams.Verbose = 0;
    SolverParams.Accuracy = 1e-12;
    SolverParams.Tolerance = 1e-6;
    SolverParams.Max_Inner_Iter = 100;
    SolverParams.Num_pre_funs = 10;
    SolverParams.InitSteptype = 0;
    SolverParams.IsStopped = @(x, fs, ngf, ngf0)IsStopped(x, fs, ngf, ngf0, A, M, full(rhs), SolverParams.Tolerance);
    precond = 1;
    initialY.main = Z_mess;
%     Z = randn(n,p);
%     initialY.main = Z / norm(Z, 'fro');
    [FinalY, fv, gfv, gfgf0, iter, nf, ng, nR, nV, nVp, nH, ComTime, funs, grads, times, eigHess] =...
                LyapunovLR(A, M, LM, B, rhs, n, p, ManiParams, SolverParams, HasHHR, initialY, precond);
    numSystemsb_mixed = FinalY.numSystemsb;
    numSystemsB_mixed = FinalY.numSystemsB;
    timeOfSystems_mixed = timeOfSystems_mixed + FinalY.timeOfSystems;
    time_mixed = time_mixed + toc(tt);

    FF = full([A * FinalY.main,  M * FinalY.main, rhs]);
    [~, R] = qr(FF, 0);
    p = size(FinalY.main, 2);
    l = size(rhs, 2);
    T = [zeros(p, p), eye(p), zeros(p, l); eye(p), zeros(p, p), zeros(p, l); zeros(l, p), zeros(l, p), -eye(l)]; 
    rel_res_mixed = norm(R * T * R', 'fro') / norm(rhs' * rhs, 'fro');
    
    clear enq opts oper
    

    %% RLyap-RNewton
    fprintf('\nStart RLyap-RNewton...\n')
    tt = tic;
    % common parameters for different solvers    
    IntParams.Min_Iteration = 2;
    IntParams.Max_Iteration = 100;
    IntParams.OutputGap = 1;
    IntParams.Verbose = 0;
    IntParams.Stop_Criterion = 2;
    
    precond = 1; % 0, 1, 2, 3, 4
    gap = 0;     % 0 or 3;
    ParamSet = precond + gap;    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                             RNewton                                     %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    IntParams.method = 'RNewton';
    IntParams.InitSteptype = 0;
    IntParams.ParamSet = ParamSet;
    IntParams.precond = precond;
    IntParams.Num_pre_funs = 10;
    IntParams.Tolerance = 1.e-2;
    
    p = 2;
%     [U, S, V] = svd(Z_rksm);
%     ExtParams.initialY = U(:,1:p)*S(1:p,1:p)*V(:,1:p)';
    
    ExtParams.ext_Tol = 1e-6; 
    ExtParams.numLs = 20;
    ExtParams.p_Min = p;
    ExtParams.p_Inc = 5;
    ExtParams.p_Max = 100;
    
    [FinalY_RNewton, outInfo_RNewton, rel_res_Rlyap_RNewton] = RLyapSolver(A, M, LM, B, rhs, ExtParams, IntParams);
    time_RNewton = time_RNewton + toc(tt);
    Z_RLyap_RNewton = FinalY_RNewton.main;
    numSystemsb_RNewton = outInfo_RNewton.numSystemsb;
    numSystemsB_RNewton = outInfo_RNewton.numSystemsB;
    timeOfSystems_RNewton = timeOfSystems_RNewton + outInfo_RNewton.timeOfSystems;

end

time_kpik = time_kpik / NNN;
time_rksm = time_rksm / NNN;
time_mess = time_mess / NNN;
time_mixed = time_mixed / NNN;
time_RNewton = time_RNewton / NNN;

timeOfSystems_kpik = timeOfSystems_kpik / NNN;
timeOfSystems_rksm = timeOfSystems_rksm / NNN;
timeOfSystems_mixed = timeOfSystems_mixed / NNN;
timeOfSystems_RNewton = timeOfSystems_RNewton / NNN;

    filID = fopen('./test_table4_' + data + '.txt', 'w');    
    fprintf(filID,'============================================================\n');
    fprintf(filID,'%15s %8d x %8d\n', 'Matrix size:', size(A,1), size(A,2));
    fprintf(filID,'%15s %8d\n', 'Size of rhs:', size(rhs,2)) ;
    fprintf(filID,'%15s %12s %12s %12s %12s %12s\n', 'Methods', 'rank', 'time(s.)', 'rel_res', 'numSystems','timeSystems');
    fprintf(filID,'%15s %12d $%10s$ $%10s$ %12d $%10s$\n', 'kpik', length(rel_res_kpik)*2, outputfloat(time_kpik), outputfloat(rel_res_kpik(end)), length(rel_res_kpik) + 1, outputfloat(timeOfSystems_kpik));
    fprintf(filID,'%15s %12d $%10s$ $%10s$ %12d $%10s$\n', 'rksm', length(rel_res_rksm), outputfloat(time_rksm), outputfloat(rel_res_rksm(end)), length(rel_res_rksm) + 1, outputfloat(timeOfSystems_rksm));
    fprintf(filID,'%15s %12d $%10s$ $%10s$ %12d $%10s$\n', 'mess_lradi', size(Z_mess_,2), outputfloat(time_mess), outputfloat(rel_res_mess), size(Z_mess_,2) * 2,' ');
    fprintf(filID,'%15s %12d $%10s$ $%10s$ %12d(%d) $%10s$\n', 'RLyap-RNewton', size(Z_RLyap_RNewton,2), outputfloat(time_RNewton), outputfloat(rel_res_Rlyap_RNewton(end)), numSystemsb_RNewton,numSystemsB_RNewton, outputfloat(timeOfSystems_RNewton));
    fprintf(filID,'%15s %12d $%10s$ $%10s$ %12d(%d) $%10s$\n', 'lradi-RNewton', size(FinalY.main,2), outputfloat(time_mixed), outputfloat(rel_res_mixed), numSystemsb_mixed, numSystemsB_mixed,outputfloat(timeOfSystems_mixed));    

end


% customed stopping criterion
function output = IsStopped(x, fs, ngf, ngf0, A, M, C, tol)
tic;
global res_
    Y = x.main; 
    [~, R] = qr(full([A * Y, M * Y, C]), 0);
    k = size(Y, 2);
    l = size(C, 2);
    T = [zeros(k, k), eye(k), zeros(k, l); eye(k), zeros(k, k), zeros(k, l); zeros(l, k), zeros(l, k), -eye(l)];
    residual = norm(R * T * R', 'fro') / norm(C' * C, 'fro');                
    if residual < res_
        res_ = residual;
    end
    output = (residual < tol) || (res_ < residual);

time = toc;
fprintf(">>n:%d, rank:%d, res:%.2e, res_time:%.2e, nY:%.2e\n", size(Y,1), k, residual, time, norm(Y,'fro'));    
end